/*
 * \file FortuneServer.cpp
 *
 *  \date Oct 16, 2013
 *  \author Alexandre Tuleu
 */

#include <biorob-cpp/net/Server.h>

#include <signal.h>
#include <iostream>
#include <cstdlib>


class Application {
public :
	typedef std::tr1::shared_ptr<Application> Ptr;
	Application();
	~Application();
	void Run();
private :
	net::Server d_server;
};


static Application::Ptr app;

void SigHandler(int);
void SigHandler(int sig) {
	if(sig == SIGPIPE) {
		std::cerr << "Got a broken PIPE." << std::endl;
	}
	app = Application::Ptr();
	exit(sig == SIGPIPE ? 42 : 0);
}


int main() {
	signal(SIGINT,&SigHandler);
	signal(SIGPIPE,&SigHandler);

	int ret = 0;
	try {
		app = Application::Ptr(new Application());
		app->Run();
	} catch (const std::exception & e) {
		std::cerr << "Got error : " << e.what() << std::endl;
		ret = 1;
	}

	app = Application::Ptr();
	return ret;
}


Application::Application()
	: d_server(net::Server::TCP){
}

Application::~Application(){
}

#ifdef DNS_SD_SUPPORTED
#include <biorob-cpp/net/dns-sd/Publisher.h>
#include <pthread.h>
#include <biorob-cpp/os/SystemError.h>

class ThreadCloser {
public :
	void operator()(pthread_t * t) {
		p_call_noerrno(pthread_cancel,*t);
		p_call_noerrno(pthread_join,*t,NULL);
		delete t;
	}
};

void *PublishThread(void*);
void * PublishThread(void *) {
	using namespace net::dnssd;
	Publisher p("BiorobFortunerServer");
	p.AddService(IF_UNSPEC,PROTO_UNSPEC,"_bcpp-greeter._tcp",12345);
	p.EnterMainLoop();
	return NULL;
}
#endif


void Application::Run() {
	d_server.Listen("",12345);

#ifdef DNS_SD_SUPPORTED
	std::tr1::shared_ptr<pthread_t> thread(new pthread_t,ThreadCloser());
	pthread_create(thread.get(),NULL,&PublishThread,NULL);
#endif

	std::cout << "Server is Listening" << std::endl;
	while(true) {
		net::Socket::Ptr s = d_server.Accept();
		std::cout << "Got one connection" << std::endl;
		if(!fork()) {
			s->WriteAll((uint8_t*)"Hello world!",13);
			exit(0);
		}
	}
}
