/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Config.cpp
 *
 *  \date Dec 3, 2012
 *  \author Alexandre Tuleu
 */

#include "Config.h"
#include "Parsers.h"
#include <biorob-cpp/os/Pathname.h>

#include <fstream>
#include <stdexcept>


namespace options {
Config::Config(const std::string & basename)
	: d_filename(CorrectTheBasename(basename))
	, d_stdFilenames()
	, d_root(new options::Section("","")){

	d_stdFilenames.reserve(3);
	d_stdFilenames.push_back("/etc/" + d_filename );
	d_stdFilenames.push_back(std::string(getenv("HOME")) + "/." + d_filename );
	d_stdFilenames.push_back(d_filename );
}

Config::~Config(){
}


void Config::LoadFile(std::istream & in){
	ConfigFileParser p;
	p.Parse(in,d_root);
}

void Config::LoadFile(const std::string & filename){
	ConfigFileParser p;
	p.Parse(filename,d_root);
}

void Config::AppendFile(const std::string & filename){
	d_stdFilenames.push_back(filename);
}

void Config::LoadAllFiles(Level l){
	if(l == ALL){
		l = (Level)(d_stdFilenames.size() - 1);
	}
	if((unsigned int)l >= d_stdFilenames.size() ){
		std::ostringstream os;
		os << "Level " << l << " is out of range ([ 0 , "
		   << (d_stdFilenames.size() - 1) << " ].";
		throw std::out_of_range(os.str());
	}
	for (unsigned int i = 0; i <= (unsigned int)l; ++i){
		LoadFile(d_stdFilenames[i]);
	}
}

std::string Config::CorrectTheBasename(const std::string & basename){
	os::Pathname path(basename);

	if(path.Suffix() == ".config" ){
		return path.Filename();
	} else {
		return path.Basename() + ".config";
	}
}

void Config::Print(std::ostream & out) const{
	ConfigFileParser p;
	p.Print(out,d_root);

}

void Config::Print(const std::string & filename ) const{
	std::ofstream file(filename.c_str());

	if(!file.is_open()){
		throw std::runtime_error("Could not open file '" + filename + "'.");
	}

	Print(file);

}

const std::string & Config::SystemConfigFilename()    const{
	return d_stdFilenames[SYSTEM];
}
const std::string & Config::GlobalConfigFilename()    const{
	return d_stdFilenames[GLOBAL];
}
const std::string & Config::DirectoryConfigFilename() const{
	return d_stdFilenames[DIRECTORY];
}

} //namespace options
