/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file ConfigUTest.cpp
 *
 *  \date Dec 3, 2012
 *  \author Alexandre Tuleu
 */

#include "ConfigUTest.h"
#include "Config.h"

class BarSection : public options::ConfigSection {
public :
	BarSection(options::Section::Ptr s)
		: options::ConfigSection(s)
		, d_hSomeStuff(Section()){}
	BCPP_ADD_OPTION(SomeStuff,"some-stuff",std::string,"baz","this is some stuff");
};

class FooSection : public options::ConfigSection{
public :
	FooSection(options::Section::Ptr s)
		: options::ConfigSection(s)
		, d_shBar(Section())
		, d_hSomeStuff(Section()){}
	BCPP_ADD_SUBSECTION(BarSection,Bar,"bar","This is the bar section")
	BCPP_ADD_OPTION(SomeStuff,"some-stuff",double,69.0,"this is some stuff")

};

class BazSection : public options::ConfigSection {
public :
	BazSection(options::Section::Ptr s)
		: options::ConfigSection(s)
		, d_hSomeStuff(Section())
		, d_hSomeStuff2(Section()){}
	BCPP_ADD_OPTION(SomeStuff,"some-stuff",double,69.0,"this is some stuff")
	BCPP_ADD_OPTION(SomeStuff2,"some-stuff2",double,69.0,"this is some stuff")
};


class DynamicSection : public options::DynamicConfigSection<BazSection> {
public :
	DynamicSection(options::Section::Ptr s)
		: options::DynamicConfigSection<BazSection>(s)
		, d_hTest(Section()){};
	BCPP_ADD_OPTION(Test,"test",double,69.0,"this is some test.")
	//NO BCPP_ADD_SUBSECTION
};


class MyConfig : public options::Config {
public :
	MyConfig()
		: options::Config("my-config.config")
		, d_shFoo(Section())
		, d_shBaz(Section())
		, d_shBar(Section())
		, d_shDynamic(Section()){}

	BCPP_ADD_SUBSECTION(FooSection,Foo,"foo","This is the foo section");
	BCPP_ADD_SUBSECTION(BazSection,Baz,"baz","This is the bat section");
	BCPP_ADD_SUBSECTION(BazSection,Bar,"bar","This is the bar section");
	BCPP_ADD_SUBSECTION(DynamicSection,Dynamic,"dynamic","thios is a dynamic section")
};


void ConfigUTest::SetUp(){

	d_configFileContent << "[foo]" << std::endl
	                    << "some-stuff = 42.0"<< std::endl
	                    << "[foo.bar]"<< std::endl
	                    << "some-stuff = bar" << std::endl
	                    << "[dynamic]" << std::endl
	                    << "test = 32.0" << std::endl
	                    << "[dynamic.first]" << std::endl
	                    << "some-stuff = 1.0" << std::endl
	                    << "some-stuff2 = 2.0" << std::endl
	                    << "[dynamic.second]" << std::endl
	                    << "some-stuff = 3.0" << std::endl
	                    << "some-stuff2 = 4.0" << std::endl
	                    << std::endl;

}

TEST_F(ConfigUTest,ConstructionIsCorrect){
	MyConfig c;
	EXPECT_EQ(1,c.Foo().Section().Options().size());
	EXPECT_EQ(c.Foo().Section().GetOption("some-stuff"),c.Foo().SomeStuffOption());
}


TEST_F(ConfigUTest,DefaultAreConserved){
	MyConfig c;
	EXPECT_FLOAT_EQ(69.0,c.Foo().SomeStuff());
	EXPECT_EQ("baz",c.Foo().Bar().SomeStuff());
}

TEST_F(ConfigUTest,ParsingIsOk){
	MyConfig c;
	c.LoadFile(d_configFileContent);
	EXPECT_FLOAT_EQ(42.0,c.Foo().SomeStuff());
	EXPECT_EQ("bar",c.Foo().Bar().SomeStuff());
	EXPECT_FLOAT_EQ(32.0,c.Dynamic().Test());
	ASSERT_EQ(2,c.Dynamic().SubSections().size());
	EXPECT_TRUE(c.Dynamic().SubSection("first"));
	EXPECT_TRUE(c.Dynamic().SubSection("second"));
	EXPECT_FLOAT_EQ(1.0,c.Dynamic().SubSection("first")->SomeStuff());
	EXPECT_FLOAT_EQ(2.0,c.Dynamic().SubSection("first")->SomeStuff2());
	EXPECT_FLOAT_EQ(3.0,c.Dynamic().SubSection("second")->SomeStuff());
	EXPECT_FLOAT_EQ(4.0,c.Dynamic().SubSection("second")->SomeStuff2());

}

