/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Parsers.h
 *
 *  \date Nov 28, 2011
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CPP_OPTIONS_PARSERS_H_
#define BIOROB_CPP_OPTIONS_PARSERS_H_


#include <string>
#include <iostream>
#include <list>

#include "Option.h"


namespace options{
	class OptionBase;

	class ConfigFileParser {
	public :
		ConfigFileParser();
		void Parse(const std::string & filename, Section::Ptr root);

		void Parse(std:: istream & stream, Section::Ptr root);


		//void Parse(const std::string fileBase, Config & config);

		std::ostream & Print(std::ostream & out,const Section::Ptr & root);


	private :
		typedef std::list<std::string> StackOfString;
		void ParseLine(std::string & line);

		OptionBase::Ptr GetOptionOrCreate(const std::string & name);

		Section::Ptr GetSectionOrCreate(const std::string & name);


		void SplitCommentInLines(std::string comment,
		                         unsigned int maxLineLength,
		                         std::vector<std::string> & lines);


		void StripWhitespace(std::string & string);

		void AppendNewLineToComment(const std::string & line);

		void ClearCommentStack();
		void PushNewCommentToStack();

		void SetCommentFromStack(options::SectionAndOptionBase & s);

		StackOfString  d_comments;


		Section::Ptr d_currentSection;
		Section::Ptr d_rootSection;
		std::string  d_currentSectionName;
	};


	class ArgumentParser {
	public :
		void Parse(int & argc, char ** argv, OptionBase & root);
	};
}

#endif // BIOROB_CPP_OPTIONS_PARSERS_H_
