/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * PathnameUTest.cpp
 *
 *  Created on: 7 nov. 2012
 *      Author: akay
 */

#include "PathnameUTest.h"
#include "Pathname.h"

using namespace os;

TEST_F(PathnameUTest,CanDetectAbsolutePaths){
	typedef std::vector<std::pair<std::string,bool> > PathAndRelativeness;
	PathAndRelativeness paths;

	paths.push_back(std::make_pair("./Foo/Bar/Baz",true));
	paths.push_back(std::make_pair("Foo/Bar/Baz",true));
	paths.push_back(std::make_pair("../Blah",true));
	paths.push_back(std::make_pair("/Foo/Bar/Baz",false));

	for(PathAndRelativeness::const_iterator p = paths.begin();
	    p != paths.end();
	    ++p){
		Pathname pa(p->first);
		if(p->second){
			EXPECT_TRUE(pa.IsRelative()) << pa.CompletePath() << " should be relative";
			EXPECT_FALSE(pa.IsAbsolute()) << pa.CompletePath() << " should be relative";
		} else {
			EXPECT_FALSE(pa.IsRelative()) << pa.CompletePath() << " should be absolute";;
			EXPECT_TRUE(pa.IsAbsolute()) << pa.CompletePath() << " should be absolute";;
		}
	}
	Pathname pNull;
	EXPECT_FALSE(pNull.IsAbsolute());
	EXPECT_FALSE(pNull.IsRelative());
}


TEST_F(PathnameUTest,CouldExtractFilename){
	typedef std::vector<std::pair<std::string,std::string> > PathAndNames;
	PathAndNames paths;
	paths.push_back(std::make_pair("",""));
	paths.push_back(std::make_pair("./baz/Foo.bar","Foo.bar"));
	paths.push_back(std::make_pair("Baz.bar","Baz.bar"));
	paths.push_back(std::make_pair("/Foo.bar/","Foo.bar"));

	for(PathAndNames::const_iterator p = paths.begin();
	    p != paths.end();
	    ++p){
		Pathname pa(p->first);
		EXPECT_EQ(p->second,pa.Filename());
	}
}

TEST_F(PathnameUTest,CouldExtractBasenameAndSuffix){
	typedef std::vector<std::pair<std::string, std::pair<std::string, std::string> > > PathBaseAndSuffix;

	PathBaseAndSuffix paths;
	paths.push_back(std::make_pair("",std::make_pair("","")));
	paths.push_back(std::make_pair("./foo/bar.txt",std::make_pair("bar","txt")));
	paths.push_back(std::make_pair("./foo.bar.txt",std::make_pair("foo.bar","txt")));
	paths.push_back(std::make_pair("./foo/barbar",std::make_pair("barbar","")));
	paths.push_back(std::make_pair("./foo/.bashrc",std::make_pair(".bashrc","")));
	paths.push_back(std::make_pair("./foo/.bashrc.bak",std::make_pair(".bashrc","bak")));

	for(PathBaseAndSuffix::const_iterator p = paths.begin();
	    p != paths.end();
	    ++p){
		Pathname pa(p->first);
		EXPECT_EQ(p->second.first,pa.Basename()) << "When processing '" << p->first << "'";
		EXPECT_EQ(p->second.second,pa.Suffix()) << "When processing '" << p->first << "'";
	}
}

TEST_F(PathnameUTest,CouldDetectPrefix){
	Pathname p("../foo/libMyAwesomeLib.so");
	EXPECT_FALSE(p.HasPrefix("foo"));
	EXPECT_TRUE(p.HasPrefix("lib"));
	Pathname pNull;
	//Everyone has prefix ""
	EXPECT_TRUE(p.HasPrefix(""));
	//Excepted ""
	EXPECT_FALSE(pNull.HasPrefix(""));
}
