/*
 * \file systemerror.h
 *
 *  \date Sep 18, 2013
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CPP_SYSTEMERROR_H_
#define BIOROB_CPP_SYSTEMERROR_H_

#include <errno.h>

#include <stdexcept>

#include <string>

namespace os {

class SystemError : public std::runtime_error {
public:
	SystemError(int error);
	SystemError(const std::string & reason, int error);
	virtual ~SystemError() throw();

	int Error() const;
private :
	std::string StringifyError(int error);

	int d_error;
};

/**
 * A wrapper to standard posix syscall, were the syscal should return >= 0
 * or a negative value on error, and errno sets. It will throw a SystemError. This
 * is notably how open, close function works.
 */
#define p_call(fnct,...) do{ \
	int sys_call_res ## fnct = fnct(__VA_ARGS__); \
	if ( sys_call_res ## fnct < 0 ) { \
		throw os::SystemError( #fnct + std::string("() returned with error"),errno); \
	} \
}while(0)

/**
 * A wrapper to a posix interface where 0 is returned on success , or the
 * return code otherwise. It will throw a SystemError on failure. This notably
 * the way pthread function works.
 */

#define p_call_noerrno(fnct,...) do{\
	int sys_call_res ## fnct = fnct(__VA_ARGS__);\
	if( sys_call_res ## fnct != 0 ) {\
		throw os::SystemError( #fnct + std::string("() returned with error"), sys_call_res ## fnct);\
	} \
}while(0)


} /* namespace os */
#endif /* BIOROB_CPP_SYSTEMERROR_H_ */
